"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Generated by CoffeeScript 2.4.1
(function () {
  var HighlandPagination, Promise, ReaderHistoricalProcessor, Repository, SIZE_PAGE, _, debug, highland, moment, normalizeError;

  _ = require("lodash");

  normalizeError = require("./normalize.error");

  Promise = require("bluebird");

  highland = require("highland");

  HighlandPagination = require("highland-pagination");

  moment = require("moment");

  debug = require("debug")("historical-deadletter:reader");

  Repository = require("./repository");

  require("highland-concurrent-flatmap");

  SIZE_PAGE = 100;

  module.exports = ReaderHistoricalProcessor = function () {
    function ReaderHistoricalProcessor(opts) {
      _classCallCheck(this, ReaderHistoricalProcessor);

      this.run = this.run.bind(this);
      this._retrieveFailedNotifications = this._retrieveFailedNotifications.bind(this);
      this._doProcess = this._doProcess.bind(this);
      this.processor = opts.processor;
      this.app = opts.app;
      this.job = opts.job;
      var _opts$daysRetrying = opts.daysRetrying;
      this.daysRetrying = _opts$daysRetrying === undefined ? 1 : _opts$daysRetrying;
      var _opts$concurrency = opts.concurrency;
      this.concurrency = _opts$concurrency === undefined ? {
        callsToApi: 20,
        callsToAzure: 50
      } : _opts$concurrency;
      this.conditions = opts.conditions;
      var _opts$logger = opts.logger;
      this.logger = _opts$logger === undefined ? debug : _opts$logger;

      this.repository = new Repository(opts);
    }

    _createClass(ReaderHistoricalProcessor, [{
      key: "run",
      value: function run() {
        var _this = this;

        return new HighlandPagination(this._retrieveFailedNotifications).stream().map(function (message) {
          return _.update(message, "notification", JSON.parse);
        }).concurrentFlatMap(this.concurrency.callsToApi, function (message) {
          return _this._doProcess(message).map(function () {
            return message;
          }).errors(function (err) {
            debug("Still fails " + message.app + "-" + message.job + "-" + message.resource);
            return _this.repository.save(_.merge(message, {
              notification: JSON.stringify(message.notification)
            }, normalizeError(err)));
          });
        }).tap(function (row) {
          return debug("Process successful " + row.app + "-" + row.job + "-" + row.resource);
        }).batch(20).concurrentFlatMap(this.concurrency.callsToAzure, function (it) {
          return highland(_this.repository.remove(_.map(it, "id")));
        }).reduce(0, function (accum) {
          return accum + 1;
        }).toPromise(Promise).tap(function () {
          return debug("Done process");
        });
      }
    }, {
      key: "_retrieveFailedNotifications",
      value: function _retrieveFailedNotifications(page) {
        return this.repository.search(this._conditions(), {
          page: page,
          size: SIZE_PAGE
        }).spread(function (items) {
          return {
            items: items,
            nextToken: (items != null ? items.length : void 0) === SIZE_PAGE ? page + 1 : void 0
          };
        });
      }
    }, {
      key: "_conditions",
      value: function _conditions() {
        var nDaysAgo;
        if (this.conditions != null) {
          return this.conditions;
        }
        nDaysAgo = moment().subtract(this.daysRetrying, 'days').utc().format("YYYY-MM-DDTHH:mm:ss") + "z";
        return ["app eq '" + this.app + "'", "job eq '" + this.job + "'", "timestamp gt " + nDaysAgo];
      }
    }, {
      key: "_doProcess",
      value: function _doProcess(row) {
        var _this2 = this;

        return highland(function (push, next) {
          var __done;
          __done = function __done(err) {
            push(err, null);
            return push(null, highland.nil);
          };
          return _this2.processor({
            done: __done,
            log: _this2.logger
          }, row);
        });
      }
    }]);

    return ReaderHistoricalProcessor;
  }();
}).call(undefined);